﻿local SN, L, F, T, M, _ = unpack(select(2,...))
local LSM = LibStub("LibSharedMedia-3.0")
local MODNAME = "raidinvite"
local RI = SN:NewModule(MODNAME, "AceEvent-3.0", "AceConsole-3.0", "AceTimer-3.0")

local guildMembers = {}
local InviteQueue = {}
local Invites = {}
local masterLooter = ""
local maintank = ""
local inviteStart = false

function RI:BuildFilterTable()
    local _, str

    local globalStrings = {
		JOINED_PARTY,
		ERR_INVITE_PLAYER_S,
		ERR_RAID_MEMBER_ADDED_S,
		ERR_NEW_LOOT_MASTER_S,
		ERR_SET_LOOT_MASTER,
		ERR_SET_LOOT_THRESHOLD_S,
    }

    RI.filterTable = {}

    for _,v in ipairs(globalStrings) do
        str = string.gsub(v, "%.", "")
        str = string.gsub(str, "%%s", ".+")
        RI.filterTable[#RI.filterTable+1] = str
    end
end

function RI:ChatFilter(self, event, msg, ...)
    if not RI.filterTable then RI:BuildFilterTable() end

    for _, v in ipairs(RI.filterTable) do
        if string.find(msg, v) then return 1 end
    end

    return nil
end

function RI:OnInitialize()
	--Inviter frame to allow OnUpdate
	RI.Inviter = CreateFrame("Frame")
	--Raid Inviter option table for AceConfigDialog
	RI.options = {
		type = "group",
		name = "Guild Raid Inviter",
		childGroups = "tree",
		args = {
			memberSelect = {
				type = "group",
				name = "Guild Member List",
				guiInline = true,
				args = {
					invite = {
						order = 0,
						type = "execute",
						name = "Invite",
						func = function()
							if inviteStart then
								SN:Print("The Inviter has already started")
							else
								inviteStart = true
								local i = 1
								for k, v in pairs(InviteQueue) do
									Invites[i] = k
									i = i + 1
								end
								RI:Invite()
							end
						end,
					},
					refresh = {
						order = 1,
						type = "execute",
						name = "Refresh",
						func = function()
							RI:RefreshGuildData()
						end,
					},
					masterloot = {
						order = 2,
						name = "Master Looter",
						type = "select",
						values = function()
							local values = {}
							for k, v in pairs(guildMembers) do
								values[k] = k
							end
							return values
						end,
						get = function(info)
							return masterLooter
						end,
						set = function(info, value)
							masterLooter = value
						end,		
					},
					tank = {
						order = 3,
						name = "Main Tank",
						type = "select",
						values = function()
							local values = {}
							for k, v in pairs(guildMembers) do
								values[k] = k
							end
							return values
						end,
						get = function(info)
							return maintank
						end,
						set = function(info, value)
							maintank = value
						end,
					},
					members = {
						order = 4,
						type = "multiselect",
						name = "",
						values = function()
							local values = {}
							for k, v in pairs(guildMembers) do
								values[k] = k
							end
							return values
						end,
						get = function(info, value)
							return guildMembers[value]
						end,
						set = function(info, value, bool)
							guildMembers[value] = bool
							if bool then
								InviteQueue[value] = { name = value, status = 2 }
							else
								if tContains(InviteQueue, value) then
									SN:tremovebyval(InviteQueue, value)
								end
								if SN.data.profile.debug.enable then
									for k, v in pairs(InviteQueue) do
										SN:Print(k.." "..v)
									end
								end
							end
						end,
					},
				},
			},
		},
	}

	--Raid Invite Tool
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Inviter", RI.options)
	LibStub("AceConfigDialog-3.0"):SetDefaultSize("Inviter", 450, 500)
	--Raid Function command line tools
	RI:RegisterChatCommand("raidinviter", "Cmds")
end

function RI:RefreshGuildData()
	for i = 1, GetNumGuildMembers() do
		local name, _, rankIndex, unitLevel, _, unitZone, _, _, online = GetGuildRosterInfo(i)
		if unitLevel == 90 and online and name ~= SN.playerName then
			if InviteQueue[name] then
				guildMembers[name] = true
			else
				guildMembers[name] = false
			end
		end
	end
end

function RI:Cmds(input)
	local v1, v2 = input:match("^(%S*)%s*(.-)$")
	v1 = v1:lower()

	if (v1 == "") then	
		LibStub("AceConfigDialog-3.0"):Open("Inviter")
	end
end

local delay = 0
local inviteProgress = 1
local MLset = false
local filtered = false

local function inviteUpdate(self, elapsed)
	if not filtered then
		ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", RI.ChatFilter)
		filtered = true
	end
	delay = delay + elapsed
	if delay > 5 then		
		if Invites[inviteProgress] then
			if inviteProgress > 3 and #Invites > 5 and not IsInRaid() then
				SN:Print("Setting group to Raid")
				ConvertToRaid()
			end
			SN:Print("Inviting..."..Invites[inviteProgress])
			InviteUnit(Invites[inviteProgress])
			inviteProgress = inviteProgress + 1
			delay = 0
		else
			local lootmethod = GetLootMethod()
			if lootmethod ~= "master" and strlen(masterLooter) ~= 0 and IsInRaid() and not MLset then
				if UnitInRaid(masterLooter) then
					SetLootMethod("master", masterLooter)
					SN:Print("Setting "..masterLooter.." to Master Looter")
					PromoteToAssistant(masterLooter)
					SN:Print("Setting "..masterLooter.." to Assistant")
					masterlooter = ""
					MLset = true
				end
				delay = 0
			elseif MLset and IsInRaid() and masterlooter == "" then
				self:SetScript("OnUpdate", nil)
				if strlen(maintank) ~= 0 and UnitInRaid(maintank) then
					PromoteToAssistant(maintank)
					SN:Print("Setting "..maintank.." to Assistant")
					maintank = ""
				end
				SetLootThreshold(4)
				SN:Print("Setting Loot Threshold to Epic")
				SN:Print("Invites Done")
				wipe(InviteQueue)
				wipe(Invites)
				inviteStart = false
				inviteProgress = 1
				delay = 0
				MLset = false
				ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", RI.ChatFilter)
				filtered = false
				InitiateRolePoll()
			end
		end
	end
	
end

function RI:Invite()
	SN:Print("Starting invites...")
	RI.Inviter:SetScript("OnUpdate", inviteUpdate)
end

function RI:OnEnable()
	
end

function RI:OnDisable()

end

function RI:Lock()

end

function RI:Unlock()
	
end

function RI:Reset()

end

function RI:ApplySettings()

end

