﻿local SN, L, F, T, M, _ = unpack(select(2,...))
local LSM = LibStub("LibSharedMedia-3.0")
local MODNAME = "raidbuffs"
local RB = SN:NewModule(MODNAME, "AceEvent-3.0", "AceConsole-3.0")

local raidMembers = {}
local UpdateDelay = 0
local raidbuffs

local function getOptions()
	raidbuffs = {
		type = "group",
		name = L["Raid Buffs"],
		args = {
			enable = {
				type = "toggle",
				name = L["Enable Raid Buff System"],
				order = 0,
				get = function(info)
					return SN.data.profile.raidbuffs[info[#info]]
				end,
				set = function(info, value)
					SN.data.profile.raidbuffs[info[#info]] = value
					if value then
						SN:EnableModule(MODNAME)
					else
						SN:DisableModule(MODNAME)
					end
				end,
			},
		},
	}
	return raidbuffs
end

local function SpellName(spellID)
	local name = GetSpellInfo(spellID)
	return name
end

local function SpellIcon(spellID)
	local _, _, icon = GetSpellInfo(spellID)
	return icon
end

local player = {}
player.__index = player
function player:new()
	self = {}
	setmetatable(self, player)
	self.name = ""
	self.flask = false
	self.food = false
	self.fortitude = false
	self.statbuff = false
	self.masterybuff = false
	self.spbuff = false
	self.critbuff = false
	
	return self
end

local buffs = {
	flask = 0,
	food = 0,
	fortitude = 0,
	statbuff = 0,
	masterybuff = 0,
	spbuff = 0,
	critbuff = 0,
}
local missingBuffs = {
	flask = {},
	food = {},
	fortitude = {},
	statbuff = {},
	masterybuff = {},
	spbuff = {},
	critbuff = {},
}

local buffspells = {
	["flask"] = {
		SpellName(105689),  -- Flask of Spring Blossoms
		SpellName(105691),  -- Flask of the Warm Sun
		SpellName(105693),  -- Flask of Falling Leaves
		SpellName(105694),  -- Flask of the Earth
		SpellName(105696),  -- Flask of Winter\'s Bite
	}, 
	["food"] = {
		SpellName(35272),	-- Well Fed
	},
	["fortitude"] = {
		SpellName(21562), 	-- Prayer of Fortitude
		SpellName(6307),	-- Blood Pact
		SpellName(469),     -- Commanding Shout
		SpellName(90364),   -- Qiraji Fortitude
		SpellName(111923),  -- Runescroll of Fortitude III
	},
	["statbuff"] = {
		SpellName(1126), 	-- Mark of the Wild
		SpellName(20217), 	-- Blessing of Kings
		SpellName(117666), 	-- Legacy of the Emperor
		SpellName(90363),   -- Embrace of the Shale Spider
	},
	["masterybuff"] = { 
		SpellName(19740),   -- Blessing of Might
		SpellName(116956),  -- Grace of Air
		SpellName(93435),   -- Roar of Courage
		SpellName(128997),  -- Spirit Beast Blessing        
	},
	["spbuff"] = { 
		SpellName(1459), 	-- Arcane Intellect
		SpellName(61316), 	-- Dalaran Brilliance
		SpellName(109773), 	-- Dark Intent
		SpellName(77747), 	-- Burning Wrath
		SpellName(126309), 	-- Still Water
	},
	["critbuff"] = {
		SpellName(116781),	-- Legacy of the White Tiger
		SpellName(17007), 	-- Leader of the Pack
		SpellName(1459), 	-- Arcane Brilliance
		SpellName(61316),	-- Dalaran Brilliance
		SpellName(24604), 	-- Furious Howl
		SpellName(90309), 	-- Terrifying Roar
		SpellName(97229), 	-- Bellowing Roar
		SpellName(126373),	-- Fearless Roar
		SpellName(126309),	-- Still Water        
	},
}

local spellIcons = {
	flask = SpellIcon(105691),
	food = SpellIcon(35272),
	fortitude = SpellIcon(21562),
	statbuff = SpellIcon(20217),
	masterybuff = SpellIcon(19740),
	spbuff = SpellIcon(1459),
	critbuff = SpellIcon(17007),
}

function player:CheckBuffs(buff)
	--Check for buffs
	for k, v in pairs(buffspells[buff]) do
		if UnitBuff(self.name, buffspells[buff][k]) then
			self[buff] = true
			return
		end
	end
end

function RB:OnInitialize()
	SN:RegisterModuleOptions(MODNAME, getOptions, L["Raid Buffs"])

	--Raid Buff Window
	RB.Buffs = CreateFrame("Frame", nil, UIParent)
	RB.Buffs:SetSize(336, 46)
	RB.Buffs:SetPoint(unpack(SN.data.profile.frames.raidbuffs))
	RB.Buffs:SetClampedToScreen(true)
	
	SN:CreateBackDrop(RB.Buffs)
	
	local counter = ""
	for k, v in pairs(buffs) do		
		RB.Buffs[k] = CreateFrame("Button", nil, RB.Buffs, "SecureActionButtonTemplate")
		RB.Buffs[k]:SetSize(32, 32)
		RB.Buffs[k]:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight")
		RB.Buffs[k]:SetNormalTexture(spellIcons[k])
		RB.Buffs[k]:SetParent(RB.Buffs)
		if counter == "" then
			RB.Buffs[k]:SetPoint("LEFT", RB.Buffs, 14, 0)
		else
			RB.Buffs[k]:SetPoint("LEFT", RB.Buffs[counter], "RIGHT", 14, 0)
		end
		RB.Buffs[k]:SetScript("OnEnter", function(self, motion)
			GameTooltip:SetOwner(self, "ANCHOR_BOTTOMLEFT")
			GameTooltip:SetText("Missing Buff:")
			if not missingBuffs[k] then
				GameTooltip:AddLine("none")
			else
				for a, b in pairs(missingBuffs[k]) do
					GameTooltip:AddLine(b)
				end
			end
			GameTooltip:Show()
		end)
		RB.Buffs[k]:SetScript("OnLeave", function()
			GameTooltip:ClearLines()
			GameTooltip:Hide()
		end)
		RB.Buffs[k]:SetScript("OnClick", function(self, button, down)
			local report
			for k, v in pairs (missingBuffs[k]) do
				if not report then
					report = k
				else
					report = strjoin(", ", report, k)
				end
			end
			if report then
				SendChatMessage("People missing "..k..": "..report, "RAID")
			else
				SendChatMessage("Buffs are all good", "RAID")
			end
		end)
		local color = NORMAL_FONT_COLOR
		RB.Buffs[k].text = RB.Buffs[k]:CreateFontString(nil, "ARTWORK")
		RB.Buffs[k].text:SetFontObject(GameFontGreenLarge )
		RB.Buffs[k].text:SetPoint("CENTER", RB.Buffs[k], "CENTER", 0, 0)
		RB.Buffs[k].text:SetText("0")
		RB.Buffs[k].text:Show()
		counter = k
		RB.Buffs[k].name = k
	end
	local color = NORMAL_FONT_COLOR
	RB.Buffs.value = RB.Buffs:CreateFontString(nil, "ARTWORK")
	RB.Buffs.value:SetFont(M.media.font, 14, "OUTLINE")
	RB.Buffs.value:SetPoint("CENTER", RB.Buffs)
	RB.Buffs.value:SetJustifyH("CENTER")
	RB.Buffs.value:SetTextColor(color.r, color.g, color.b)
	RB.Buffs.name = "Raid Buff Frame"
	RB.Buffs.db = MODNAME

	RB.Buffs:SetScript("OnEvent", function(self, event, ...)
		if event == "PLAYER_REGEN_ENABLED" then
			if IsInGroup() then
				self:Show()
				RB:ToggleActive(self, true)
			else
				RB:ToggleActive(self, false)
			end
		elseif event == "PLAYER_REGEN_DISABLED" then
			RB:ToggleActive(self, false)
			self:Hide()
		elseif event == "GROUP_ROSTER_UPDATE" then
			if not InCombatLockdown() then
				if not IsInGroup() then
					RB:ToggleActive(self, false)
					for k, v in pairs(buffs) do
						self[k].text:SetText("0")
						buffs[k] = 0
						if missingBuffs[k] then
							wipe(missingBuffs[k])
						end
					end
				else
					RB:ToggleActive(self, true)
					wipe(raidMembers)
					for i = 1, GetNumGroupMembers() do
						local rname = GetRaidRosterInfo(i)
						if rname then
							local cachename = rname:gsub("^(.*)%-.*$", "%1")
							if not raidMembers[cachename] then
								raidMembers[cachename] = player:new()
								raidMembers[cachename].name = cachename
							end
						end
					end
				end
			end
		end
	end)
	RB.Buffs:Hide()
end

function RB:OnEnable()
	if not IsInGroup() then
		RB:ToggleActive(RB.Buffs, false)
		for k, v in pairs(buffs) do
			RB.Buffs[k].text:SetText("0")
			buffs[k] = 0
			if missingBuffs[k] then
				wipe(missingBuffs[k])
			end
		end
	else
		RB:ToggleActive(RB.Buffs, true)
		wipe(raidMembers)
		for i = 1, GetNumGroupMembers() do
			local rname = GetRaidRosterInfo(i)
			if rname then
				local cachename = rname:gsub("^(.*)%-.*$", "%1")
				if not raidMembers[cachename] then
					raidMembers[cachename] = player:new()
					raidMembers[cachename].name = cachename
				end
			end
			RB:RefreshBuffs()
			for k, v in pairs(buffs) do
				RB.Buffs[k].text:SetText(tostring(buffs[k]))
			end
			UpdateDelay = 0
		end
	end
	RB.Buffs:RegisterEvent("PLAYER_REGEN_DISABLED")
	RB.Buffs:RegisterEvent("PLAYER_REGEN_ENABLED")
	RB.Buffs:RegisterEvent("GROUP_ROSTER_UPDATE")
	RB.Buffs:RegisterEvent("READY_CHECK")
end

function RB:OnDisable()
	RB.Buffs:UnregisterAllEvents()
	RB.Buffs:Hide()
	RB:ToggleActive(RB.Buffs, false)
end

function RB:Lock()
	SN:LockFrame(RB.Buffs)
	for k, v in pairs(buffs) do		
		RB.Buffs[k]:Show()
	end
	SN:CreateBackDrop(RB.Buffs)
end

function RB:Unlock()
	for k, v in pairs(buffs) do		
		RB.Buffs[k]:Hide()
	end
	SN:UnlockFrame(RB.Buffs)
end

function RB:Reset()
	local static = SN.DB.profile.frames
	local temp = SN.data.profile.frames
	RB.Buffs:ClearAllPoints()

	RB.Buffs:SetPoint(unpack(static.raidbuffs))
	wipe(temp.raidbuffs)
end

function RB:ApplySettings()
	RB.Buffs:ClearAllPoints()
	RB.Buffs:SetPoint(unpack(SN.data.profile.frames.raidbuffs))
end

function RB:RefreshBuffs()
	for k, v in pairs(buffs) do
		buffs[k] = 0
	end
	for k, v in pairs(missingBuffs) do
		wipe(missingBuffs[k])
	end
	for k, v in pairs(raidMembers) do
		for i, j in pairs(raidMembers[k]) do
			if type(v) == "boolean" then
				raidMembers[k][i] = false
			end
		end
	end

	for k, v in pairs(raidMembers) do
		for i, j in pairs(buffs) do
			raidMembers[k]:CheckBuffs(i)
		end
	end
	--SN:Print(#raidMembers)
	for i, j in pairs(raidMembers) do
		for k, v in pairs(buffs) do
			if not raidMembers[i][k] then
				buffs[k] = buffs[k] + 1
				missingBuffs[k][i] = i
			end
		end
	end
end

function RB:ToggleActive(self, bool)
	if bool then
		self:Show()
		self:SetScript("OnUpdate", function(self, elapsed)
			UpdateDelay = UpdateDelay + elapsed
			if UpdateDelay > 10 then
				if not IsInGroup() then
					for k, v in pairs(buffs) do
						self[k].text:SetText("0")
						buffs[k] = 0
						if missingBuffs[k] then
							wipe(missingBuffs[k])
						end
					end 
				else
					RB:RefreshBuffs()
					for k, v in pairs(buffs) do
						self[k].text:SetText(tostring(buffs[k]))
					end
				end
				UpdateDelay = 0
			end
		end)
	else
		self:SetScript("OnUpdate", nil)
		self:Hide()
	end
end